#!/usr/bin/python

import gtk
import gnome
import gnome.ui
import gnome.config
import libglade
import signal
import GdkImlib
import sys
import re
import os
import pwd
from gtk import TRUE
from gtk import FALSE
try:
    import cString
    string = cString
except:
    import string
import errno

# The user's selections
apply_wm = TRUE
apply_panel = FALSE



def get_home_dir ():
    entry = pwd.getpwuid (os.getuid ())

    return entry[5]

# if there is no ~/.gnome/panel file, then chances are, they've never run gnome, and we can exit
try:
    os.stat (get_home_dir() + "/.gnome/panel.d")
except:
    sys.exit ()

# check the version
runbefore = gnome.config.get_int ("/gnome-upgrade/Config/runbefore=0")
if runbefore >= 120:
    sys.exit()

gnome.config.set_int ("/gnome-upgrade/Config/runbefore", 120)
gnome.config.sync ()

class WMConfig:
    def __init__ (self):
        self.focus_mode = 'click'
        self.move_mode = 'opaque'
        self.resize_mode = 'box'
        self.tooltips = 1
        self.num_desktops = 4
        self.area_rows = 1
        self.area_columns = 1

    def spew (self):
        print "focus: " + self.focus_mode
        print "move: " + self.move_mode
        print "resize: " + self.resize_mode
        print "tooltips: " + ("%d" % self.tooltips)
        print "num_desktops: %d" % self.num_desktops
        print "virtual area rows: %d" % self.area_rows
        print "virtual area columns: %d" % self.area_columns
        

def load_e_config ():
    e_config_line = re.compile ('([0-9]+)(.*)')

    e_config_file = os.path.join (get_home_dir (), ".enlightenment/...e_session-XXXXXX")

    handle = None
    try:
        handle = open (e_config_file)
    except:
        return WMConfig ()

    conf = WMConfig ()

    lines = handle.readlines ()

    for line in lines:
        match = e_config_line.match (line)
        if match:
            magic_crackrock_code = string.strip (match.group (1))
            value = string.strip (match.group (2))

            if magic_crackrock_code == '307':
                if value == '0':
                    conf.focus_mode = 'mouse'
                elif value == '1':
                    conf.focus_mode = 'sloppy'
                elif value == '2':
                    conf.focus_mode = 'click'
                        
            elif magic_crackrock_code == '311':
                if value == '0':
                    conf.move_mode = 'opaque'
                else:
                    conf.move_mode = 'box'
            elif magic_crackrock_code == '312':
                if value == '0':
                    conf.resize_mode = 'opaque'
                else:
                    conf.resize_mode = 'box'
            elif magic_crackrock_code == '17':
                if value == '0':
                    conf.tooltips = 0
            elif magic_crackrock_code == '333':
                conf.num_desktops = string.atoi (value)
            elif magic_crackrock_code == '407':
                (columns, rows) = string.split (value, ' ')
                conf.area_rows = string.atoi (rows)
                conf.area_columns = string.atoi (columns)

    return conf

def save_sawfish_config (conf):
    dir = os.path.join (get_home_dir (), ".sawfish")
    file = os.path.join (dir, "custom")

    try:
        os.mkdir (dir)
    except OSError, e:
        if e.errno != errno.EEXIST:
            return

    ## only save a sawfish config if none existed previously
    if os.path.exists (file):
        return
    
    handle = None
    try:
        # use append mode, just for paranoia
        handle = open (file, 'a')
    except:
        return

    sawfish_focus = 'click'
    if conf.focus_mode == 'mouse':
        sawfish_focus = 'enter-exit'
    elif conf.focus_mode == 'sloppy':
        sawfish_focus = 'enter-only'

    handle.write ('(custom-set-variable (quote focus-mode) (quote %s))\n' % sawfish_focus)

    handle.write ('(custom-set-variable (quote resize-outline-mode) (quote %s))\n' % conf.resize_mode)

    handle.write ('(custom-set-variable (quote move-outline-mode) (quote %s))\n' % conf.move_mode)

    sawfish_tooltips = 't'
    if not conf.tooltips:
        sawfish_tooltips = 'nil'

    handle.write ('(custom-set-variable (quote tooltips-enabled) (quote %s) (quote tooltips))\n' % sawfish_tooltips)

    handle.write ('(custom-set-variable (quote preallocated-workspaces) (quote %s))\n' % conf.num_desktops)

    handle.write ('(custom-set-variable (quote viewport-columns) (quote %d))' % conf.area_columns)

    handle.write ('(custom-set-variable (quote viewport-rows) (quote %d))' % conf.area_rows)
    
    handle = None

def set_sawfish_default ():
    section_line = re.compile ('^\[(.*)\]')
    eline = re.compile ('([0-9]+).*enlightenment')
    in_default = FALSE
    e_priority = None

    session_file = os.path.join (get_home_dir (), ".gnome/session")
    session_out_file = os.path.join (get_home_dir (), ".gnome/session.NOE")

    in_handle = None
    out_handle = None
    try:
        in_handle = open (session_file, 'r')
        out_handle = open (session_out_file, 'w')
    except:
        return

    #find the priority in the section.
    lines = in_handle.readlines ()
    for line in lines:
        match = section_line.match (line)
        if match:
            if match.group (1) == "Default":
                in_default = TRUE
            else:
                in_default = FALSE

        if in_default == FALSE:
            continue
        match = eline.match (line)
        if match:
            e_priority = match.group (1)

    if e_priority == None:
        in_handle.close ()
        out_handle.close ()
        return

    written_session = FALSE
    re_string = ("^%s," % e_priority)
    skip_line = re.compile (re_string)
    for line in lines:
        match = section_line.match (line)
        if match:
            if match.group (1) == "Default":
                in_default = TRUE
            else:
                in_default = FALSE

        if not in_default or not skip_line.match (line):
            out_handle.write (line)
        elif (written_session == FALSE):
            written_session = TRUE
            out_handle.write ("""%s,id=default%s
%s,Priority=10
%s,RestartCommand=sawfish --sm-client-id default%s
""" % (e_priority, e_priority, e_priority, e_priority, e_priority))

    in_handle.close ()
    out_handle.close ()
    os.system ("mv %s %s" % (session_out_file, session_file))
    gnome.config.set_string ("/wm-properties/Config/Config/Current", "Sawfish")

def check_gtkrc ():
    import_line = re.compile ('include \"(.*)\"')
    themename_line = re.compile ('include \".*/(.*)/gtk/gtkrc\"')
    gtkrc = os.path.join (get_home_dir (), ".gtkrc")

    handle = None
    try:
        handle = open (gtkrc)
    except:
        return FALSE

    lines = handle.readlines ()

    for line in lines:
        match = import_line.match (line)
        if match:
            if not os.access (match.group (1), os.R_OK):
                match2 = themename_line.match (line)
                if match2:
                    return match2.group (1)
                return "Unknown Theme"
            return FALSE
    return FALSE

def grep_for_sawmill (filename):
    sawmill_re = re.compile ('sawmill')
    handle = None
    try:
         handle = open (filename)
    except:
        return FALSE

    lines = handle.readlines ()
    for line in lines:
        match = sawmill_re.search (line)
        if match:
            return TRUE

    return FALSE

def check_sawmill ():
    if grep_for_sawmill (os.path.join (get_home_dir (), ".gnome/session")):
        return TRUE

    if grep_for_sawmill (os.path.join (get_home_dir (), ".gnome/wm-properties/Config")):
        return TRUE

    return FALSE
    
def fix_sawmill ():
    session_file = os.path.join (get_home_dir (), ".gnome/session")
    wm_file = os.path.join (get_home_dir (), ".gnome/wm-properties/Config")

    if grep_for_sawmill (session_file):
	command = "/usr/bin/sawmill-replace.sh %s" % session_file
        print "Running: %s" % command
        os.system (command)

    if grep_for_sawmill (wm_file):
        command = "/usr/bin/sawmill-replace.sh %s" % wm_file
        print "Running: %s" % command
        os.system (command)

    os.system ("/bin/cp ~/.sawmillrc ~/.sawfishrc")
    os.system ("/bin/cp -a ~/.sawmill ~/.sawfish")
 

# Are we running gnome-1.0 or 1.2.  If we are running gnome-1.0, this won't have been set.
gnome10 = gnome.config.get_string ("/panel/Config/menu_key=UNSET")
if gnome10 == "UNSET":
    panel_needed = TRUE
else:
    panel_needed = FALSE

# Figure out which of the druid pages we need.
wm = gnome.config.get_string ("/wm-properties/Config/Config/Current=UNSET")
if wm == "Enlightenment":
    wm_needed = TRUE
else:
    wm_needed = FALSE

theme_needed = check_gtkrc ()

sawmill_needed = check_sawmill ()

# do we need to run the druid at all?
if (wm_needed == FALSE and panel_needed == FALSE and theme_needed == FALSE and sawmill_needed == FALSE):
    sys.exit ()

xml = libglade.GladeXML("/usr/share/gnome-upgrade/glade/gnome-upgrade.glade", "toplevel")


def on_druid_cancel (dummy):
    sys.exit ()

def on_title_page_next (page, dummy):
    druid = xml.get_widget ("druid")
    if wm_needed:
        druid.set_page (xml.get_widget ("wm_page"))
        return TRUE
    if panel_needed:
        druid.set_page (xml.get_widget ("panel_page"))
        return TRUE
    if theme_needed:
        druid.set_page (xml.get_widget ("theme_page"))
        return TRUE
    druid.set_page (xml.get_widget ("finish_page"))
    return TRUE

def on_wm_page_next (page, dummy):
    global apply_wm
    global apply_panel
    druid = xml.get_widget ("druid")
    cbutton = xml.get_widget ("sawfish_radio")
    if (cbutton.active):
        apply_wm = TRUE
    else:
        apply_wm = FALSE
    if panel_needed:
        druid.set_page (xml.get_widget ("panel_page"))
        return TRUE
    if theme_needed:
        druid.set_page (xml.get_widget ("theme_page"))
        return TRUE
    druid.set_page (xml.get_widget ("finish_page"))
    return TRUE

def on_wm_page_back (page, dummy):
    druid = xml.get_widget ("druid")
    druid.set_page (xml.get_widget ("title_page"))
    return TRUE

def on_panel_page_next (page, dummy):
    global apply_wm
    global apply_panel
    druid = xml.get_widget ("druid")
    cbutton = xml.get_widget ("panel_radio")
    if not cbutton.active:
        apply_panel = TRUE
    else:
        apply_panel = FALSE
    if theme_needed:
        druid.set_page (xml.get_widget ("theme_page"))
        return TRUE
    druid.set_page (xml.get_widget ("finish_page"))
    return TRUE

def on_panel_page_back (page, dummy):
    druid = xml.get_widget ("druid")
    if wm_needed:
        druid.set_page (xml.get_widget ("wm_page"))
        return TRUE
    druid.set_page (xml.get_widget ("title_page"))
    return TRUE

def on_theme_page_next (page, dummy):
    druid = xml.get_widget ("druid")
    druid.set_page (xml.get_widget ("finish_page"))
    return TRUE

def on_theme_page_prepare (page, dummy):
    label = xml.get_widget ("wm_warning_label")
    label.set_text ("""Your current GTK+ theme, \"%s\" is no longer shipped with the Red Hat
packages.  Your theme will be switched back to the default theme.

Your old theme is likely available at http://gtk.themes.org/""" % theme_needed)

def on_theme_page_back (page, dummy):
    druid = xml.get_widget ("druid")
    if panel_needed:
        druid.set_page (xml.get_widget ("panel_page"))
        return TRUE
    if wm_needed:
        druid.set_page (xml.get_widget ("wm_page"))
        return TRUE
    druid.set_page (xml.get_widget ("title_page"))
    return TRUE

def on_finish_page_back (page, dummy):
    druid = xml.get_widget ("druid")
    if theme_needed:
        druid.set_page (xml.get_widget ("theme_page"))
        return TRUE
    if panel_needed:
        druid.set_page (xml.get_widget ("panel_page"))
        return TRUE
    if wm_needed:
        druid.set_page (xml.get_widget ("wm_page"))
        return TRUE
    druid.set_page (xml.get_widget ("title_page"))
    return TRUE

def on_finish_page_prepare (page, dummy):
    global apply_wm
    global apply_panel
    if (not apply_wm) and (not apply_panel) and (not sawmill_needed):
        text = """You have elected to not upgrade your GNOME desktop.

Pressing \"Finish\" will continue the log-in process"""
    else:
        text = "Please press \"Finish\" to activate the following changes\n\n"
    if apply_wm:
        text = text + "     Switching window manager to sawfish\n"
    if apply_panel:
        text = text + "     Resetting panel to defaults\n"
    if sawmill_needed:
        text = text + "     Replacing 'sawmill' with 'sawfish' as your default window manager\n"
        if os.path.exists (os.path.join (get_home_dir (), '.sawmillrc')) and not os.path.exists (os.path.join (get_home_dir (), '.sawfishrc')): 
            text = text + "     Copying '~/.sawmillrc' to '~/.sawfishrc'\n"
        if os.path.exists (os.path.join (get_home_dir (), '.sawmill')) and not os.path.exists (os.path.join (get_home_dir (), '.sawfish')):
            text = text + "     Copying '~/.sawmill/' to '~/.sawfish/'\n"
    page.set_text (text)

def on_finish_page_finish (page, dummy):
    # before sed'ing the config files in fix_sawmill ()
    gnome.config.sync ()

    if sawmill_needed:
        fix_sawmill ()

    if apply_wm == TRUE:
        conf = load_e_config ()
        save_sawfish_config (conf)
        set_sawfish_default ()
    if apply_panel == TRUE:
        os.system ("rm -rf ~/.gnome/panel*")

    sys.exit ()

def main ():
    xml.signal_autoconnect (
        { "on_druid_cancel" : on_druid_cancel,
          "on_title_page_next" : on_title_page_next,
          "on_wm_page_next" : on_wm_page_next,
          "on_panel_page_next" : on_panel_page_next,
          "on_theme_page_next" : on_theme_page_next,
          "on_wm_page_back" : on_wm_page_back,
          "on_panel_page_back" : on_panel_page_back,
          "on_theme_page_prepare" : on_theme_page_prepare,
          "on_theme_page_back" : on_theme_page_back,
          "on_finish_page_back" : on_finish_page_back,
          "on_finish_page_prepare" : on_finish_page_prepare,
          "on_finish_page_finish" : on_finish_page_finish})
    window = xml.get_widget ("toplevel")
    cbutton = xml.get_widget ("sawfish_radio")
    cbutton.set_active (TRUE)
    cbutton = xml.get_widget ("panel_radio")
    cbutton.set_active (TRUE)
    window.show_all ()
    gtk.mainloop ()

if __name__ == "__main__":
    signal.signal (signal.SIGINT, signal.SIG_DFL)
    main ()





